import process from 'node:process';
import { createUnplugin } from 'unplugin';
import { createFilter } from '@rollup/pluginutils';
import { HTMLRewriter } from 'html-rewriter-wasm';

function check(code, options, length) {
    // check if fold is valid
    for (const [start, end] of options) {
        if (start > end) {
            return {
                valid: false,
                message: `Invalid input: start ${start} > end ${end}`,
            };
        }
    }
    // check if fold is in the code
    for (const [start, end] of options) {
        if (start > length || end > length) {
            return {
                valid: false,
                message: `Invalid input: start ${start} or end ${end} > length ${length}`,
            };
        }
    }
    return {
        valid: true,
        message: "",
    };
}

function processFold(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    options.forEach(([start, end]) => {
        // line start end
        // insert // ...
        const startIdx = code.findIndex((item) => item.lineNum === start);
        const endIdx = code.findIndex((item) => item.lineNum === end);
        code.splice(startIdx, endIdx - startIdx + 1, {
            lineNum: start,
            content: "// ...",
        });
    });
    return code;
}

function processTruncate(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    // sort by start
    options.sort((a, b) => a[0] - b[0]);
    let newCode = [];
    for (const [start, end] of options) {
        const startIdx = code.findIndex((item) => item.lineNum === start);
        const endIdx = code.findIndex((item) => item.lineNum === end);
        newCode = newCode.concat(code.slice(startIdx, endIdx + 1));
    }
    return newCode;
}

const encoder = new TextEncoder();
const decoder = new TextDecoder();
async function rewrite(html, opt) {
    let output = "";
    const rewriter = new HTMLRewriter((outputChunk) => {
        output += decoder.decode(outputChunk);
    });
    const { fileContent, tagNames } = opt;
    try {
        for (const tagName of tagNames) {
            rewriter.on(tagName, {
                element(element) {
                    const foldOptRaw = element.getAttribute("fold");
                    const foldOptions = foldOptRaw ? JSON.parse(foldOptRaw) : [];
                    const truncateOptRaw = element.getAttribute("truncate");
                    const truncateOptions = truncateOptRaw ? JSON.parse(truncateOptRaw) : [];
                    let metaLines = fileContent.split("\n").map((content, index) => ({
                        lineNum: index + 1,
                        content,
                    }));
                    const length = metaLines.length;
                    metaLines = processFold(metaLines, foldOptions, length);
                    metaLines = processTruncate(metaLines, truncateOptions, length);
                    const processedCode = metaLines.map((line) => line.content).join("\n");
                    const base64Content = Buffer.from(processedCode).toString("base64");
                    element.setAttribute("data-sample-code", base64Content);
                },
            });
        }
        await rewriter.write(encoder.encode(html));
        await rewriter.end();
        return output;
    }
    catch (e) {
        console.error(e);
        return html;
    }
    finally {
        rewriter.free();
    }
}
async function handleRemoveInProd(html, opt) {
    let output = "";
    const rewriter = new HTMLRewriter((outputChunk) => {
        output += decoder.decode(outputChunk);
    });
    try {
        for (const tagName of opt.tagNames) {
            rewriter.on(tagName, {
                element(element) {
                    element.remove();
                },
            });
        }
        await rewriter.write(encoder.encode(html));
        await rewriter.end();
        return output;
    }
    catch (e) {
        console.error(e);
        return html;
    }
    finally {
        rewriter.free();
    }
}

const defaultInclude = [
    "**/*.vue",
    "**/*.jsx",
    "**/*.tsx",
    "**/*.astro",
    "**/*.svelte",
    "**/*.html",
];
const defaultTagName = ["code-sample", "CodeSample", "codeSample"];
const unpluginFactory = (options = {}) => {
    return {
        name: "vite-plugin-code-sample",
        transformInclude(id) {
            return filter(options)(id);
        },
        async transform(code, id) {
            return await transform(code, id, options);
        },
    };
};
function filter(options) {
    return createFilter(options.include || defaultInclude, options.exclude);
}
async function transform(code, id, options) {
    var _a;
    const tagNames = (options === null || options === void 0 ? void 0 : options.tagName)
        ? options.tagName.concat(defaultTagName)
        : defaultTagName;
    if (tagNames.length === 0) {
        return code;
    }
    let hasMatch = false;
    tagNames.forEach((tagName) => {
        if (code.includes(tagName)) {
            hasMatch = true;
        }
    });
    if (!hasMatch) {
        return code;
    }
    const removeInProd = (_a = options === null || options === void 0 ? void 0 : options.removeInProd) !== null && _a !== void 0 ? _a : true;
    if (removeInProd && process.env.NODE_ENV === "production") {
        return await handleRemoveInProd(code, { tagNames });
    }
    return await rewrite(code, { fileContent: code, tagNames });
}
const unplugin = /* #__PURE__ */ createUnplugin(unpluginFactory);

export { unplugin as default, filter, transform, unplugin, unpluginFactory };
